/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "winwarndialog.h"
#include <QMessageBox>
#include "xmlsettingsmanager.h"

namespace LeechCraft
{
	WinWarnDialog::WinWarnDialog ()
	{
		run ();
	}
	
	void WinWarnDialog::run ()
	{
		deleteLater ();
		if (XmlSettingsManager::Instance ()->
					Property ("Win32UserAgreed", false).toBool ())
			return;
		
		const QString& text =
			tr ("Seems like you are running Microsoft Windows.\n\nLeechCraft "
				"for Windows is quite experimental and unstable, some builds "
				"may use unstable versions of third-party libraries, and, "
				"moreover, LeechCraft is known to have more bugs and less "
				"features on Windows than on other, saner operating systems. "
				"That's because we have not so much developers running Windows, "
				"and most of the users use other OSes. Sorry for that, "
				"we hope you'll understand us.\n\nIf you have read and really "
				"understood all these letters, click Cancel button in this "
				"dialog so that it won't pop up again next time you run LeechCraft.");

		if (QMessageBox::warning (0,
					"LeechCraft",
					text,
					QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Cancel)
			XmlSettingsManager::Instance ()->setProperty ("Win32UserAgreed", true);
	}
}
