/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <memory>
#include <QObject>
#include <QHash>
#include <QCache>
#include <QPointer>
#include <QSet>
#include <QImage>
#include <interfaces/core/ihookproxy.h>

class QStandardItem;

namespace LeechCraft
{
namespace Azoth
{
	class IProxyObject;
	class ICLEntry;
	struct EntryStatus;
	class AvatarsManager;

	class CLTooltipManager : public QObject
	{
		Q_OBJECT

		AvatarsManager * const AvatarsManager_;

		typedef QHash<ICLEntry*, QList<QStandardItem*>> Entry2Items_t;
		Entry2Items_t& Entry2Items_;

		QCache<ICLEntry*, QString> Avatar2TooltipSrcCache_;

		QSet<ICLEntry*> DirtyTooltips_;
	public:
		CLTooltipManager (AvatarsManager*, Entry2Items_t&);

		void AddEntry (ICLEntry*);
		void RemoveEntry (ICLEntry*);

		QString MakeTooltipString (ICLEntry*);

		void RebuildTooltip (ICLEntry *entry);
	private:
		QString MakeTooltipString (ICLEntry*, QString);
	private slots:
		void remakeTooltipForSender ();
		void handleAvatarChanged (QObject*);

		void handleCacheSizeChanged ();
		void handleAvatarsSizeChanged ();
	signals:
		void hookTooltipBeforeVariants (LeechCraft::IHookProxy_ptr proxy,
				QObject *entry) const;

		void rebuiltTooltip ();
	};
}
}
