/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "dndutil.h"
#include <QString>
#include <QList>
#include <QDataStream>
#include <QMimeData>
#include "interfaces/azoth/iclentry.h"
#include "core.h"

namespace LeechCraft
{
namespace Azoth
{
namespace DndUtil
{
	const QString CLEntryFormat = "x-leechcraft/azoth-cl-entry";

	QString GetFormatId ()
	{
		return CLEntryFormat;
	}

	QByteArray Encode (const QList<MimeContactInfo>& entries, QMimeData *data)
	{
		QByteArray encoded;
		QDataStream stream (&encoded, QIODevice::WriteOnly);
		for (const auto info : entries)
			stream << info.Entry_->GetEntryID () << info.Group_;

		if (data)
			data->setData (CLEntryFormat, encoded);

		return encoded;
	}

	QObject* DecodeEntryObj (const QMimeData *mime)
	{
		QDataStream stream (mime->data (CLEntryFormat));
		QString sid;
		stream >> sid;

		return Core::Instance ().GetEntry (sid);
	}

	QList<QObject*> DecodeEntryObjs (const QMimeData *mime)
	{
		QList<QObject*> result;
		for (const auto& info : DecodeMimeInfos (mime))
			result << info.Entry_->GetQObject ();
		return result;
	}

	QList<MimeContactInfo> DecodeMimeInfos (const QMimeData *mime)
	{
		QList<MimeContactInfo> result;

		QDataStream stream (mime->data (CLEntryFormat));
		while (!stream.atEnd ())
		{
			QString id;
			QString group;
			stream >> id >> group;

			const auto entryObj = Core::Instance ().GetEntry (id);
			const auto entry = qobject_cast<ICLEntry*> (entryObj);
			if (!entry)
				continue;

			result.append ({ entry, group });
		}

		return result;
	}

	bool HasContacts (const QMimeData *data)
	{
		return data->hasFormat (CLEntryFormat);
	}
}
}
}
