/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2010-2011 Oleg Linkin
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "channelpublicmessage.h"
#include <QtDebug>
#include <QTextDocument>
#include "channelclentry.h"
#include "channelhandler.h"

namespace LeechCraft
{
namespace Azoth
{
namespace Acetamide
{
	ChannelPublicMessage::ChannelPublicMessage (const QString& msg,
			ChannelCLEntry *entry)
	: QObject (0)
	, ParentEntry_ (entry)
	, Message_ (msg)
	, Datetime_ (QDateTime::currentDateTime ())
	, Direction_ (Direction::Out)
	, Type_ (Type::MUCMessage)
	, SubType_ (SubType::Other)
	{
	}

	ChannelPublicMessage::ChannelPublicMessage (const QString& msg,
			IMessage::Direction direction,
			ChannelCLEntry *entry,
			IMessage::Type type,
			IMessage::SubType subType,
			ChannelParticipantEntry_ptr part)
	: QObject (0)
	, ParentEntry_ (entry)
	, ParticipantEntry_ (part)
	, Message_ (msg)
	, Datetime_ (QDateTime::currentDateTime ())
	, Direction_ (direction)
	, Type_ (type)
	, SubType_ (subType)
	{
	}

	QObject* ChannelPublicMessage::GetQObject ()
	{
		return this;
	}

	void ChannelPublicMessage::Send ()
	{
		if (!ParentEntry_)
			return;

		ParentEntry_->GetChannelHandler ()->SendPublicMessage (Message_);
	}

	void ChannelPublicMessage::Store ()
	{
		if (!ParentEntry_)
			return;

		ParentEntry_->HandleMessage (this);
	}

	IMessage::Direction ChannelPublicMessage::GetDirection () const
	{
		return Direction_;
	}

	IMessage::Type ChannelPublicMessage::GetMessageType () const
	{
		return Type_;
	}

	void ChannelPublicMessage::SetMessageType (IMessage::Type t)
	{
		Type_ = t;
	}

	IMessage::SubType ChannelPublicMessage::GetMessageSubType () const
	{
		return SubType_;
	}

	void ChannelPublicMessage::SetMessageSubType
			(IMessage::SubType type)
	{
		SubType_ = type;
	}

	QObject* ChannelPublicMessage::OtherPart () const
	{
		switch (Direction_)
		{
		case Direction::In:
			return ParticipantEntry_.get ();
		case Direction::Out:
			return ParentEntry_;
		}

		qWarning () << Q_FUNC_INFO
				<< "unknown direction"
				<< static_cast<int> (Direction_);
		return ParentEntry_;
	}

	QObject* ChannelPublicMessage::ParentCLEntry () const
	{
		return ParentEntry_;
	}

	QString ChannelPublicMessage::GetOtherVariant () const
	{
		return qobject_cast<ICLEntry*> (OtherPart ())->GetEntryName ();
	}

	QString ChannelPublicMessage::GetBody () const
	{
		return Message_;
	}

	void ChannelPublicMessage::SetBody (const QString& msg)
	{
		Message_ = msg;
	}

	QDateTime ChannelPublicMessage::GetDateTime () const
	{
		return Datetime_;
	}

	void ChannelPublicMessage::SetDateTime (const QDateTime& dt)
	{
		Datetime_ = dt;
	}
};
};
};
