/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "keeso.h"
#include <QIcon>

namespace LeechCraft
{
namespace Azoth
{
namespace Keeso
{
	void Plugin::Init (ICoreProxy_ptr)
	{
		Commands_.append ({
				{ "/keeso" },
				[] (ICLEntry*, QString& text) -> bool
				{
					text = text.mid (QString ("/keeso ").length ()).trimmed ();
					bool isUpper = qrand () % 2;
					for (int i = 0, length = text.length (); i < length; ++i)
					{
						const auto& c = text.at (i);
						const auto& u = c.toUpper ();
						const auto& l = c.toLower ();
						if (u == l)
							continue;

						text [i] = isUpper ? u : l;
						isUpper = (qrand () % 4) ? !isUpper : isUpper;
					}

					return false;
				},
				tr ("Randomily changes the capitalization of outbound messages."),
				{}
			});
	}

	void Plugin::SecondInit ()
	{
	}

	QByteArray Plugin::GetUniqueID () const
	{
		return "org.LeechCraft.Azoth.Keeso";
	}

	void Plugin::Release ()
	{
	}

	QString Plugin::GetName () const
	{
		return "Azoth Keeso";
	}

	QString Plugin::GetInfo () const
	{
		return tr ("Allows one to WrItE lIkE tHiS easily.");
	}

	QIcon Plugin::GetIcon () const
	{
		static QIcon icon ("lcicons:/azoth/keeso/resources/images/keeso.svg");
		return icon;
	}

	QSet<QByteArray> Plugin::GetPluginClasses () const
	{
		QSet<QByteArray> result;
		result << "org.LeechCraft.Plugins.Azoth.Plugins.IGeneralPlugin";
		return result;
	}

	StaticCommands_t Plugin::GetStaticCommands (ICLEntry*)
	{
		return Commands_;
	}
}
}
}

LC_EXPORT_PLUGIN (leechcraft_azoth_keeso, LeechCraft::Azoth::Keeso::Plugin);
