/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <QString>
#include <QStringList>
#include <QXmppDataForm.h>

namespace LeechCraft
{
namespace Azoth
{
namespace Xoox
{
	class AdHocCommand
	{
		QString Name_;
		QString Node_;
	public:
		AdHocCommand (const QString& name, const QString& node);

		QString GetName () const;
		void SetName (const QString&);

		QString GetNode () const;
		void SetNode (const QString&);
	};

	class AdHocNote
	{
	public:
		enum class Severity
		{
			Info,
			Warn,
			Error
		};
	private:
		Severity Severity_;
		QString Text_;
	public:
		AdHocNote (const QDomElement&);
		AdHocNote (const QString& severity, const QString& text);
		AdHocNote (Severity severity, const QString& text);

		Severity GetSeverity () const;
		const QString& GetText () const;
	};

	class AdHocResult
	{
		QString Node_;
		QString SessionID_;

		QXmppDataForm Form_;

		QStringList Actions_;

		QList<AdHocNote> Notes_;
	public:
		QString GetNode () const;
		void SetNode (const QString&);

		QString GetSessionID () const;
		void SetSessionID (const QString&);

		QXmppDataForm GetDataForm () const;
		void SetDataForm (const QXmppDataForm&);

		QStringList GetActions () const;
		void SetActions (const QStringList&);

		const QList<AdHocNote>& GetNotes () const;
		void AddNote (const AdHocNote&);
	};
}
}
}
