/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <QObject>
#include <interfaces/azoth/iprotocol.h>
#include <interfaces/azoth/imucprotocol.h>
#include <interfaces/azoth/iurihandler.h>
#include <interfaces/azoth/isupportimport.h>

namespace LeechCraft
{
struct Entity;

namespace Azoth
{
class IProxyObject;

namespace Xoox
{
	class GlooxAccount;
	class CapsDatabase;
	class VCardStorage;

	class GlooxProtocol : public QObject
						, public IProtocol
						, public IMUCProtocol
						, public IURIHandler
						, public ISupportImport
	{
		Q_OBJECT
		Q_INTERFACES (LeechCraft::Azoth::IProtocol
				LeechCraft::Azoth::IMUCProtocol
				LeechCraft::Azoth::IURIHandler
				LeechCraft::Azoth::ISupportImport)

		QObject *ParentProtocolPlugin_;
		QList<GlooxAccount*> Accounts_;

		CapsDatabase *CapsDB_;
		VCardStorage *VCardStorage_;
		IProxyObject *ProxyObject_ = nullptr;
	public:
		GlooxProtocol (CapsDatabase*, VCardStorage*, QObject* = nullptr);
		virtual ~GlooxProtocol ();

		void Prepare ();

		IProxyObject* GetProxyObject () const;
		void SetProxyObject (IProxyObject*);

		CapsDatabase* GetCapsDatabase () const;
		VCardStorage* GetVCardStorage () const;

		QObject* GetQObject ();
		ProtocolFeatures GetFeatures () const;
		QList<QObject*> GetRegisteredAccounts ();
		QObject* GetParentProtocolPlugin () const;
		QString GetProtocolName () const;
		QIcon GetProtocolIcon () const;
		QByteArray GetProtocolID () const;
		QList<QWidget*> GetAccountRegistrationWidgets (AccountAddOptions);
		void RegisterAccount (const QString&, const QList<QWidget*>&);
		void RemoveAccount (QObject*);

		QWidget* GetMUCJoinWidget ();
		QVariantMap TryGuessMUCIdentifyingData (const QString&, QObject*);

		bool SupportsURI (const QUrl&) const;
		void HandleURI (const QUrl&, QObject*);

		QString GetImportProtocolID () const;
		bool ImportAccount (const QVariantMap&);
		QString GetEntryID (const QString&, QObject*);
	private:
		void RestoreAccounts ();
	private slots:
		void saveAccounts () const;
	signals:
		void accountAdded (QObject*);
		void accountRemoved (QObject*);
	};
}
}
}
