/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#ifndef PLUGINS_AZOTH_PLUGINS_XOOX_JABBERSEARCHMANAGER_H
#define PLUGINS_AZOTH_PLUGINS_XOOX_JABBERSEARCHMANAGER_H
#include <QSet>
#include <QXmppClientExtension.h>

namespace LeechCraft
{
namespace Azoth
{
namespace Xoox
{
	class JabberSearchManager : public QXmppClientExtension
	{
		Q_OBJECT

		QSet<QString> FieldRequests_;
		QSet<QString> SearchRequests_;
	public:
		struct Item
		{
			QMap<QString, QString> Dictionary_;

			Item ();
			Item (const QString& jid,
					const QString& first, const QString& last,
					const QString& nick, const QString& email);
		};

		bool handleStanza (const QDomElement&);

		void RequestSearchFields (const QString& server);

		void SubmitSearchRequest (const QString& server, QXmppElement);
		void SubmitSearchRequest (const QString& server, const QList<QXmppElement>&);
		void SubmitSearchRequest (const QString& server, const QXmppDataForm&);
	private:
		bool CheckError (const QDomElement&);
		QList<Item> FromForm (const QDomElement&);
		QList<Item> FromStandardItems (const QDomElement&);
	signals:
		void gotSearchFields (const QString& server, const QXmppElement& containing);
		void gotItems (const QString& server, const QList<JabberSearchManager::Item>& items);
		void gotServerError (const QXmppIq& iq);
	};
}
}
}

#endif
