/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#ifndef PLUGINS_AZOTH_PLUGINS_XOOX_PUBSUBMANAGER_H
#define PLUGINS_AZOTH_PLUGINS_XOOX_PUBSUBMANAGER_H
#include <boost/function.hpp>
#include <QXmppClientExtension.h>
#include "pepeventbase.h"

namespace LeechCraft
{
namespace Azoth
{
namespace Xoox
{
	class PEPEventBase;

	class PubSubManager : public QXmppClientExtension
	{
		Q_OBJECT

		typedef boost::function<PEPEventBase* ()> Creator_t;
		QMap<QString, Creator_t> Node2Creator_;

		QMap<QString, bool> AutosubscribeNodes_;
	public:
		template<typename T>
		void RegisterCreator ()
		{
			RegisterCreator (T::GetNodeString (), StandardCreator<T>);
		}
		void RegisterCreator (const QString&, boost::function<PEPEventBase* ()>);

		template<typename T>
		void SetAutosubscribe (bool enabled)
		{
			SetAutosubscribe (T::GetNodeString (), enabled);
		}
		void SetAutosubscribe (const QString&, bool);

		void PublishEvent (PEPEventBase*);

		void RequestItem (const QString& jid,
				const QString& node, const QString& id);

		QStringList discoveryFeatures () const;
		bool handleStanza (const QDomElement& elem);
	private:
		bool HandleIq (const QDomElement&);
		bool HandleMessage (const QDomElement&);
		void ParseItems (QDomElement, const QString&);
	signals:
		void gotEvent (const QString&, PEPEventBase*);
	};
}
}
}

#endif
