/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "useravatardata.h"
#include <QDomElement>
#include <QBuffer>
#include <QCryptographicHash>
#include <QXmppElement.h>

namespace LeechCraft
{
namespace Azoth
{
namespace Xoox
{
	const QString NsAvatarDataNode = "urn:xmpp:avatar:data";

	UserAvatarData::UserAvatarData (const QImage& image)
	: Img_ (image)
	{
		{
			QBuffer buffer (&Saved_);
			buffer.open (QIODevice::WriteOnly);
			image.save (&buffer, "PNG", 0);
		}

		Hash_ = QCryptographicHash::hash (Saved_, QCryptographicHash::Sha1).toHex ();
	}

	QString UserAvatarData::GetNodeString ()
	{
		return NsAvatarDataNode;
	}

	QXmppElement UserAvatarData::ToXML () const
	{
		QXmppElement item;
		item.setTagName ("item");
		item.setAttribute ("id", Hash_);

		QXmppElement data;
		data.setTagName ("data");
		data.setAttribute ("xmlns", NsAvatarDataNode);
		data.setValue (Saved_.toBase64 ());

		item.appendChild (data);

		return item;
	}

	void UserAvatarData::Parse (const QDomElement& elem)
	{
		Hash_ = elem.attribute ("id").toLatin1 ();

		const QDomElement& data = elem.firstChildElement ("data");
		Saved_ = QByteArray::fromBase64 (data.text ().toLatin1 ().trimmed ());

		Img_ = QImage::fromData (Saved_);
	}

	QString UserAvatarData::Node () const
	{
		return NsAvatarDataNode;
	}

	PEPEventBase* UserAvatarData::Clone () const
	{
		return new UserAvatarData (*this);
	}

	QImage UserAvatarData::GetImage () const
	{
		return Img_;
	}
}
}
}
