/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "notifymanager.h"
#include <QTimer>
#include <interfaces/core/icoreproxy.h>
#include <interfaces/core/ientitymanager.h>
#include "core.h"

namespace LeechCraft
{
namespace BitTorrent
{
	NotifyManager::NotifyManager (QObject *parent)
	: QObject (parent)
	, IsReady_ (false)
	{
	}

	void NotifyManager::PluginsAvailable()
	{
		QTimer::singleShot (3000,
				this,
				SLOT (makeDelayedReady ()));
	}

	void NotifyManager::AddNotification (const Entity& e)
	{
		if (!IsReady_)
			Queue_ << e;
		else
			SendNotification (e);
	}

	void NotifyManager::SendNotification (const Entity& e)
	{
		Core::Instance ()->GetProxy ()->GetEntityManager ()->HandleEntity (e);
	}

	void NotifyManager::makeDelayedReady ()
	{
		IsReady_ = true;
		for (const auto& e : Queue_)
			SendNotification (e);
		Queue_.clear ();
	}
}
}
