/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include <cmath>
#include <QFileInfo>
#include <QDirIterator>
#include <QtDebug>
#include <util/util.h>
#include "thirdstep.h"
#include "secondstep.h"
#include "newtorrentwizard.h"

namespace LeechCraft
{
namespace BitTorrent
{
	ThirdStep::ThirdStep (QWidget *parent)
	: QWizardPage (parent)
	, TotalSize_ (0)
	{
		setupUi (this);
		registerField ("PieceSize", PieceSize_);
		registerField ("URLSeeds", URLSeeds_);
		registerField ("DHTEnabled", DHTEnabled_);
		registerField ("DHTNodes", DHTNodes_);
	}

	void ThirdStep::initializePage ()
	{
		TotalSize_ = 0;
		QString path = field ("RootPath").toString ();

		QFileInfo pathInfo (path);
		if (pathInfo.isDir ())
		{
			QDirIterator it (path,
					QDirIterator::Subdirectories);
			while (it.hasNext ())
			{
				it.next ();
				QFileInfo info = it.fileInfo ();
				if (info.isFile () && info.isReadable ())
					TotalSize_ += info.size ();
			}
		}
		else if (pathInfo.isFile () &&
				pathInfo.isReadable ())
			TotalSize_ += pathInfo.size ();

		quint64 max = std::log (static_cast<long double> (TotalSize_ / 102400)) * 80;

		quint32 pieceSize = 32 * 1024;
		int shouldIndex = 0;
		for (; TotalSize_ / pieceSize >= max; pieceSize *= 2, ++shouldIndex) ;

		if (shouldIndex > PieceSize_->count () - 1)
			shouldIndex = PieceSize_->count () - 1;

		PieceSize_->setCurrentIndex (shouldIndex);

		on_PieceSize__currentIndexChanged ();
	}

	void ThirdStep::on_PieceSize__currentIndexChanged ()
	{
		quint32 mul = 32 * 1024;
		int index = PieceSize_->currentIndex ();
		while (index--)
			mul *= 2;

		int numPieces = TotalSize_ / mul;
		if (TotalSize_ % mul)
			++numPieces;

		NumPieces_->setText (QString::number (numPieces) +
				tr (" pieces (%1)")
				.arg (LeechCraft::Util::MakePrettySize (TotalSize_)));
	}
}
}
