/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2010-2012  Oleg Linkin
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "addaccountwizardfirstpage.h"
#include <QtDebug>
#include "interfaces/blogique/ibloggingplatform.h"
#include "interfaces/blogique/iaccount.h"
#include "core.h"

namespace LeechCraft
{
namespace Blogique
{
	AddAccountWizardFirstPage::AddAccountWizardFirstPage (QWidget *parent)
	: QWizardPage (parent)
	{
		Ui_.setupUi (this);

		connect (Ui_.BloggingPlatformBox_,
				SIGNAL (currentIndexChanged (int)),
				this,
				SLOT (readdWidgets ()));

		connect (Ui_.RegisterAccount_,
				SIGNAL (toggled (bool)),
				this,
				SLOT (readdWidgets ()));

		connect (Ui_.NameEdit_,
				SIGNAL (textChanged (const QString&)),
				this,
				SLOT (handleAccountNameChanged (const QString&)));
	}

	void AddAccountWizardFirstPage::initializePage ()
	{
		registerField ("AccountName*", Ui_.NameEdit_);
		registerField ("AccountBloggingPlatform", Ui_.BloggingPlatformBox_);
		registerField ("RegisterNewAccount", Ui_.RegisterAccount_);

		const QList<IBloggingPlatform*>& platforms = Core::Instance ()
				.GetBloggingPlatforms ();

		Q_FOREACH (IBloggingPlatform *platform, platforms)
			Ui_.BloggingPlatformBox_->addItem (platform->GetBloggingPlatformIcon (),
					platform->GetBloggingPlatformName (),
					QVariant::fromValue<QObject*> (platform->GetQObject ()));

		connect (wizard (),
				SIGNAL (accepted ()),
				this,
				SLOT (handleAccepted ()));
	}

	bool AddAccountWizardFirstPage::isComplete () const
	{
		const auto& accs = Core::Instance ().GetAccounts ();
		const auto& name = Ui_.NameEdit_->text ();
		return std::find_if (accs.begin (), accs.end (),
				[&name] (decltype (accs.front ()) acc)
					{ return acc->GetAccountName () == name; }) == accs.end ();
	}

	void AddAccountWizardFirstPage::readdWidgets ()
	{
		const int idx = Ui_.BloggingPlatformBox_->currentIndex ();
		if (idx == -1)
			return;

		QObject *obj = Ui_.BloggingPlatformBox_->itemData (idx).value<QObject*> ();
		IBloggingPlatform *platform = qobject_cast<IBloggingPlatform*> (obj);
		if (!platform)
		{
			qWarning () << Q_FUNC_INFO
					<< "unable to cast"
					<< obj
					<< "to IBloggingPlatform";
			return;
		}

		Ui_.RegisterAccount_->setEnabled (platform->GetFeatures () &
				(IBloggingPlatform::BPFSupportsRegistration));

		const int currentId = wizard ()->currentId ();
		Q_FOREACH (const int id, wizard ()->pageIds ())
			if (id > currentId)
				wizard ()->removePage (id);
		qDeleteAll (Widgets_);

		IBloggingPlatform::AccountAddOptions opts = IBloggingPlatform::AAONoOptions;
		if (Ui_.RegisterAccount_->isChecked ())
			opts |= IBloggingPlatform::AAORegisterNewAccount;

		Widgets_ = platform->GetAccountRegistrationWidgets (opts, Ui_.NameEdit_->text ());
		if (!Widgets_.size ())
			return;

		const QString& platformName = platform->GetBloggingPlatformName ();
		Q_FOREACH (QWidget *widget, Widgets_)
		{
			QWizardPage *page = qobject_cast<QWizardPage*> (widget);
			if (!page)
			{
				page = new QWizardPage (wizard ());
				page->setTitle (tr ("%1 options")
						.arg (platformName));
				page->setLayout (new QVBoxLayout ());
				page->layout ()->addWidget (widget);
			}
			wizard ()->addPage (page);
		}

		setFinalPage (false);
	}

	void AddAccountWizardFirstPage::handleAccepted ()
	{
		QObject *obj = Ui_.BloggingPlatformBox_->
				itemData (field ("AccountBloggingPlatform")
						.toInt ()).value<QObject*> ();
		IBloggingPlatform *ibp = qobject_cast<IBloggingPlatform*> (obj);
		if (!ibp)
		{
			qWarning () << Q_FUNC_INFO
					<< "unable to cast"
					<< obj
					<< "to IBloggingPlatform";
			return;
		}

		ibp->RegisterAccount (Ui_.NameEdit_->text (), Widgets_);
	}

	void AddAccountWizardFirstPage::handleAccountNameChanged (const QString&)
	{
		emit completeChanged ();
		readdWidgets ();
	}
}
}
