/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2010-2012  Oleg Linkin
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "sendmessagedialog.h"
#include <QMessageBox>
#include <QPushButton>
#include <util/util.h>
#include "ljaccount.h"
#include "ljprofile.h"

namespace LeechCraft
{
namespace Blogique
{
namespace Metida
{
	SendMessageDialog::SendMessageDialog (LJProfile *profile, QWidget *parent) 
	: QDialog (parent)
	, Account_ (0)
	, Profile_ (profile)
	{
		Ui_.setupUi (this);
		Account_ = qobject_cast<LJAccount*> (Profile_->GetParentAccount ());
		Ui_.ButtonBox_->addButton (tr ("Send"), QDialogButtonBox::AcceptRole);
	}
	
	void SendMessageDialog::accept ()
	{
		if (Ui_.Addresses_->text ().isEmpty ())
		{
			QMessageBox::warning (this, "LeechCraft",
					tr ("Please enter a valid username"));
			return;
		}
		else if (Account_)
		{
			const auto& addresses = GetAddresses ();
			if (addresses.count () == 1 && addresses.at (0) == Account_->GetOurLogin ())
			{
				QMessageBox::warning (this, "LeechCraft",
					tr ("Stop trying to message yourself, livejournal is not that kind of service"));
				return;
			}
		}

		QDialog::accept ();
	}

	QStringList SendMessageDialog::GetAddresses () const
	{
		return Ui_.Addresses_->text ().split (',');
	}

	void SendMessageDialog::SetAddresses (const QStringList& addresses)
	{
		Ui_.Addresses_->setText (addresses.join (","));
	}

	QString SendMessageDialog::GetSubject () const
	{
		return Ui_.Subject_->text ();
	}

	QString SendMessageDialog::GetText () const
	{
		return Ui_.Text_->toPlainText ();
	}
}
}
}
