/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2010-2013  Oleg Linkin
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "submittodialog.h"
#include <QStandardItemModel>
#include "interfaces/blogique/iaccount.h"
#include "interfaces/blogique/ibloggingplatform.h"
#include "core.h"
#include "selecttargetdelegate.h"

namespace LeechCraft
{
namespace Blogique
{
	SubmitToDialog::SubmitToDialog (QWidget *parent)
	: QDialog (parent)
	, Model_ (new QStandardItemModel (this))
	{
		Ui_.setupUi (this);

		Model_->setHorizontalHeaderLabels ({tr ("Account"), tr ("Target") });
		Ui_.TargetsView_->setModel (Model_);

		Ui_.TargetsView_->setItemDelegateForColumn (SubmitToDialog::Target, new SelectTargetDelegate (this));

		for (auto acc : Core::Instance ().GetAccounts ())
		{
			QStandardItem *accountItem = new QStandardItem (acc->GetAccountName ());
			accountItem->setEditable (false);
			accountItem->setCheckable (true);

			QStandardItem *targetItem = new QStandardItem;

			Model_->appendRow ({ accountItem, targetItem });
			Ui_.TargetsView_->resizeColumnToContents (Account);
			Item2Account_ [accountItem] = acc;

			Ui_.TargetsView_->openPersistentEditor (Model_->indexFromItem (targetItem));
		}
	}

	IAccount* SubmitToDialog::GetAccountFromIndex (const QModelIndex& index) const
	{
		return Item2Account_.value (Model_->itemFromIndex (index));
	}

	QAbstractItemModel* SubmitToDialog::GetModel () const
	{
		return Model_;
	}

	QList<QPair<IAccount*, QString>> SubmitToDialog::GetPostingTargets () const
	{
		QList<QPair<IAccount*, QString>> result;
		for (int i = 0; i < Model_->rowCount (); ++i)
		{
			auto accItem = Model_->item (i, Account);
			auto targetItem = Model_->item (i, Target);
			if (accItem->checkState () != Qt::Checked)
				continue;

			auto account = Item2Account_.value (accItem);
			auto ibp = qobject_cast<IBloggingPlatform*> (account->GetParentBloggingPlatform ());
			if (!ibp)
				continue;

			QString target;
			if (ibp->GetFeatures () & IBloggingPlatform::BPFSelectablePostDestination)
				target = targetItem->data (SelectTargetDelegate::TargetRole).toString ();
			result << qMakePair<IAccount*, QString> (account, target);
		}

		return result;
	}

}
}
