/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "tasks.h"
#include <QAbstractItemModel>
#include <interfaces/ijobholder.h>
#include <interfaces/iinfo.h>
#include <interfaces/core/icoreproxy.h>
#include <interfaces/core/ipluginsmanager.h>
#include "core.h"

namespace LeechCraft
{
namespace DBusManager
{
	Tasks::Tasks (QObject *parent)
	: QObject (parent)
	{
	}

	QStringList Tasks::GetHolders () const
	{
		QObjectList plugins = Core::Instance ().GetProxy ()->
			GetPluginsManager ()->GetAllCastableRoots<IJobHolder*> ();
		QStringList result;
		Q_FOREACH (QObject *plugin, plugins)
			result << qobject_cast<IInfo*> (plugin)->GetName ();
		return result;
	}

	int Tasks::RowCount (const QString& name) const
	{
		QObjectList plugins = Core::Instance ().GetProxy ()->
			GetPluginsManager ()->GetAllCastableRoots<IJobHolder*> ();
		Q_FOREACH (QObject *plugin, plugins)
		{
			if (qobject_cast<IInfo*> (plugin)->GetName () != name)
				continue;

			QAbstractItemModel *model =
				qobject_cast<IJobHolder*> (plugin)->GetRepresentation ();

			return model->rowCount ();
		}

		throw tr ("Not found job holder %1.")
			.arg (name);
	}

	QVariantList Tasks::GetData (const QString& name, int r, int role) const
	{
		QObjectList plugins = Core::Instance ().GetProxy ()->
			GetPluginsManager ()->GetAllCastableRoots<IJobHolder*> ();
		Q_FOREACH (QObject *plugin, plugins)
		{
			if (qobject_cast<IInfo*> (plugin)->GetName () != name)
				continue;

			QAbstractItemModel *model =
				qobject_cast<IJobHolder*> (plugin)->GetRepresentation ();

			QVariantList result;
			for (int i = 0, size = model->columnCount ();
					i < size; ++i)
				result << model->index (r, i).data (role);
			return result;
		}

		throw tr ("Not found job holder %1.")
			.arg (name);
	}
}
}
