/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "intermutko.h"
#include <QIcon>
#include <QNetworkRequest>
#include <xmlsettingsdialog/xmlsettingsdialog.h>
#include <util/util.h>
#include "xmlsettingsmanager.h"
#include "acceptlangwidget.h"
#include "localeentry.h"

namespace LeechCraft
{
namespace Intermutko
{
	void Plugin::Init (ICoreProxy_ptr)
	{
		qRegisterMetaType<QList<LocaleEntry>> ("QList<LocaleEntry>");
		qRegisterMetaTypeStreamOperators<QList<LocaleEntry>> ();

		Util::InstallTranslator ("intermutko");

		XSD_ = std::make_shared<Util::XmlSettingsDialog> ();
		XSD_->RegisterObject (&XmlSettingsManager::Instance (), "intermutkosettings.xml");

		AcceptLangWidget_ = new AcceptLangWidget;
		XSD_->SetCustomWidget ("AcceptLangWidget", AcceptLangWidget_);
	}

	void Plugin::SecondInit ()
	{
	}

	QByteArray Plugin::GetUniqueID () const
	{
		return "org.LeechCraft.Intermutko";
	}

	void Plugin::Release ()
	{
	}

	QString Plugin::GetName () const
	{
		return "Intermutko";
	}

	QString Plugin::GetInfo () const
	{
		return tr ("Provides fine-grained tuning for the Accept-Language HTTP header.");
	}

	QIcon Plugin::GetIcon () const
	{
		return {};
	}

	QSet<QByteArray> Plugin::GetPluginClasses () const
	{
		QSet<QByteArray> result;
		result << "org.LeechCraft.Core.Plugins/1.0";
		return result;
	}

	Util::XmlSettingsDialog_ptr Plugin::GetSettingsDialog () const
	{
		return XSD_;
	}

	void Plugin::hookNAMCreateRequest (LeechCraft::IHookProxy_ptr proxy,
			QNetworkAccessManager*, QNetworkAccessManager::Operation*, QIODevice**)
	{
		auto req = proxy->GetValue ("request").value<QNetworkRequest> ();
		if (!req.url ().scheme ().startsWith ("http"))
			return;

		const auto& localeStr = AcceptLangWidget_->GetLocaleString ();
		req.setRawHeader ("Accept-Language",
				localeStr.isEmpty () ?
						" " :
						localeStr.toUtf8 ());
		proxy->SetValue ("request", QVariant::fromValue (req));
	}
}
}

LC_EXPORT_PLUGIN (leechcraft_intermutko, LeechCraft::Intermutko::Plugin);
