/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <memory>
#include <QObject>
#include <QHash>
#include <QVector>

class QStandardItem;
class QNetworkConfiguration;
class QNetworkConfigurationManager;
class QAbstractItemModel;
class QStandardItemModel;

class QNetworkSession;
typedef std::shared_ptr<QNetworkSession> QNetworkSession_ptr;

namespace LeechCraft
{
namespace Lemon
{
	class TrafficManager : public QObject
	{
		Q_OBJECT

		QStandardItemModel *Model_;
		QNetworkConfigurationManager *ConfManager_;

		struct InterfaceInfo
		{
			QString Name_;

			QStandardItem *Item_;
			qint64 PrevRead_;
			qint64 PrevWritten_;

			QNetworkSession_ptr LastSession_;

			QVector<qint64> DownSpeeds_;
			QVector<qint64> UpSpeeds_;

			InterfaceInfo (QStandardItem *item = 0)
			: Item_ (item)
			, PrevRead_ (0)
			, PrevWritten_ (0)
			{
			}
		};
		QHash<QString, InterfaceInfo> ActiveInterfaces_;
	public:
		TrafficManager (QObject* = 0);

		QAbstractItemModel* GetModel () const;

		QVector<qint64> GetDownHistory (const QString&) const;
		QVector<qint64> GetUpHistory (const QString&) const;

		int GetBacktrackSize () const;
	private slots:
		void addConfiguration (const QNetworkConfiguration&);
		void removeConfiguration (const QNetworkConfiguration&);
		void handleConfigChanged (const QNetworkConfiguration&);

		void updateCounters ();
	signals:
		void updated ();
	};
}
}
