/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <memory>
#include <QStringList>

namespace LeechCraft
{
namespace LMP
{
	struct TranscodingParams;

	class Format
	{
	public:
		virtual QString GetFormatID () const = 0;
		virtual QString GetFileExtension () const;
		virtual QString GetFormatName () const = 0;
		virtual QString GetCodecName () const = 0;
		virtual QString GetCodecID () const;

		enum class BitrateType
		{
			VBR,
			CBR
		};
		virtual QList<BitrateType> GetSupportedBitrates () const = 0;
		virtual QList<int> GetBitrateLabels (BitrateType) const = 0;

		virtual QStringList ToFFmpeg (const TranscodingParams&) const;
	protected:
		void StandardQualityAppend (QStringList&, const TranscodingParams&) const;
	};
	typedef std::shared_ptr<Format> Format_ptr;

	class Formats
	{
		QList<Format_ptr> Formats_;
		QList<Format_ptr> EnabledFormats_;

		static QString S_FFmpegCodecs_;
	public:
		Formats ();

		QList<Format_ptr> GetFormats () const;
		Format_ptr GetFormat (const QString&) const;
	};
}
}
