/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <memory>
#include <QtGlobal>

#if QT_VERSION < 0x050000
#include <QGraphicsWidget>
#else
#include <QQuickItem>
#endif

#if QT_VERSION < 0x050000
class QX11EmbedContainer;
#endif

namespace LeechCraft
{
namespace Mellonetray
{
#if QT_VERSION < 0x050000
	class IconHandler : public QGraphicsWidget
#else
	class IconHandler : public QQuickItem
#endif
	{
		Q_OBJECT
		Q_PROPERTY (unsigned long wid READ GetWID WRITE SetWID NOTIFY widChanged)

#if QT_VERSION < 0x050000
		std::shared_ptr<QX11EmbedContainer> Proxy_;
#else
		std::shared_ptr<QWindow> Proxy_;
#endif
		unsigned long WID_ = 0;
	public:
#if QT_VERSION < 0x050000
		IconHandler (QGraphicsItem* = 0);
#else
		IconHandler (QQuickItem* = 0);
#endif
		~IconHandler ();

		ulong GetWID () const;
		void SetWID (const ulong&);

#if QT_VERSION < 0x050000
		void setGeometry (const QRectF&) override;
#endif
	protected:
#if QT_VERSION < 0x050000
		QVariant itemChange (GraphicsItemChange change, const QVariant& value) override;
#else
		void geometryChanged (const QRectF&, const QRectF&) override;
#endif
	private:
		void Free ();
	signals:
		void widChanged ();
	};
}
}
