/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2010-2011  Oleg Linkin
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#ifndef PLUGINS_POSHUKU_PLUGINS_ONLINEBOOKMARKS_IBOOKMARKSSERVICE_H
#define PLUGINS_POSHUKU_PLUGINS_ONLINEBOOKMARKS_IBOOKMARKSSERVICE_H

#include <QFlag>
#include <QIcon>
#include <QVariant>
#include <QDateTime>

namespace LeechCraft
{
namespace Poshuku
{
namespace OnlineBookmarks
{

	class IAccount;

	class IBookmarksService
	{
	public:
		virtual ~IBookmarksService () {};

		enum Feature
		{
			FNone = 0,

			FCanRegisterAccount = 0x01
		};
		Q_DECLARE_FLAGS (Features, Feature)

		//
		virtual Features GetFeatures () const = 0;

		virtual QObject* GetQObject () = 0;

		//
		virtual QString GetServiceName () const = 0;

		//
		virtual QIcon GetServiceIcon () const = 0;

		//
		virtual QWidget* GetAuthWidget () = 0;

		//
		virtual void CheckAuthData (const QVariantMap&) = 0;

		//
		virtual void RegisterAccount (const QVariantMap&) = 0;

		//
		virtual void UploadBookmarks (QObject*, const QVariantList&) = 0;

		//
		virtual void DownloadBookmarks (QObject*, const QDateTime& from = QDateTime ()) = 0;

		//
		virtual void saveAccounts () const = 0;

		//
		virtual void removeAccount (QObject*) = 0;

		//
		virtual void accountAdded (QObjectList) = 0;

		//
		virtual void gotBookmarks (QObject*, const QVariantList&) = 0;

		//
		virtual void bookmarksUploaded () = 0;
	};
}
}
}

Q_DECLARE_OPERATORS_FOR_FLAGS (LeechCraft::Poshuku::OnlineBookmarks::IBookmarksService::Features)
Q_DECLARE_INTERFACE (LeechCraft::Poshuku::OnlineBookmarks::IBookmarksService,
					 "org.Deviant.LeechCraft.Poshuku.OnlineBookmarks.IBookmarksService/1.0");

#endif // PLUGINS_POSHUKU_PLUGINS_ONLINEBOOKMARKS_IBOOKMARKSSERVICE_H
