/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "xbelparser.h"
#include <stdexcept>
#include <QObject>
#include <QDomDocument>
#include "core.h"

namespace LeechCraft
{
namespace Poshuku
{
	XbelParser::XbelParser (const QByteArray& data)
	{
		QDomDocument document;
		QString errorString;
		int errorLine, errorColumn;
		if (!document.setContent (data, true,
					&errorString, &errorLine, &errorColumn))
			throw std::runtime_error (qPrintable (QObject::tr ("XML parse "
							"error<blockquote>%1</blockquote>at %2:%3.")
						.arg (errorString)
						.arg (errorLine)
						.arg (errorColumn)));

		QDomElement root = document.documentElement ();
		if (root.tagName () != "xbel")
			throw std::runtime_error (qPrintable (QObject::tr ("Not an XBEL entity.")));
		else if (root.hasAttribute ("version") &&
				root.attribute ("version") != "1.0")
			throw std::runtime_error (qPrintable (QObject::tr ("This XBEL is not 1.0.")));

		QDomElement child = root.firstChildElement ("folder");
		while (!child.isNull ())
		{
			ParseFolder (child);
			child = child.nextSiblingElement ("folder");
		}
	}

	void XbelParser::ParseFolder (const QDomElement& element, QStringList previous)
	{
		QString tag = element.firstChildElement ("title").text ();
		if (!tag.isEmpty () && !previous.contains (tag))
			previous << tag;

		QDomElement child = element.firstChildElement ();
		while (!child.isNull ())
		{
			if (child.tagName () == "folder")
				ParseFolder (child, previous);
			else if (child.tagName () == "bookmark")
				Core::Instance ().GetFavoritesModel ()->
					addItem (child.firstChildElement ("title").text (),
							child.attribute ("href"),
							previous);

			child = child.nextSiblingElement ();
		}
	}
}
}
