/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <QtGlobal>
#if QT_VERSION < 0x050000
#include <QDeclarativeView>
#else
#include <QQuickWidget>
#endif

#include <interfaces/core/icoreproxy.h>

class QStandardItemModel;
class QMainWindow;
class QTimer;

namespace LeechCraft
{
namespace Util
{
	class UnhoverDeleteMixin;
}

namespace SB2
{
#if QT_VERSION < 0x050000
	class TabListView : public QDeclarativeView
#else
	class TabListView : public QQuickWidget
#endif
	{
		Q_OBJECT

		ICoreProxy_ptr Proxy_;
		ICoreTabWidget * const ICTW_;
		QMainWindow * const MW_;

		const QByteArray TC_;
		QStandardItemModel * const Model_;

		Util::UnhoverDeleteMixin *UnhoverDeleteMixin_;
	public:
		TabListView (const QByteArray&, const QList<QWidget*>&,
				ICoreTabWidget*, QMainWindow*, ICoreProxy_ptr, QWidget* = 0);
		~TabListView ();

		QByteArray GetTabClass () const;

		void HandleLauncherHovered ();
		void HandleLauncherUnhovered ();
	private slots:
		void handleTabRemoved (QWidget*);
		void switchToItem (int);
		void closeItem (int);
	};
}
}
