/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "urloperator.h"

namespace LeechCraft
{
namespace Util
{
	UrlOperator::UrlOperator (QUrl& url)
	: Url_ (url)
#if QT_VERSION >= 0x050000
	, Query_ (url)
#endif
	{
	}

	UrlOperator::~UrlOperator ()
	{
		Flush ();
	}

	void UrlOperator::Flush ()
	{
#if QT_VERSION >= 0x050000
		Url_.setQuery (Query_);
#endif
	}

	UrlOperator& UrlOperator::operator() (const QString& key, const QString& value)
	{
#if QT_VERSION >= 0x050000
		Query_.addQueryItem (key, value);
#else
		Url_.addQueryItem (key, value);
#endif

		return *this;
	}

	UrlOperator& UrlOperator::operator() (const QString& key, const QByteArray& value)
	{
		return (*this) (key, QString::fromUtf8 (value));
	}

	UrlOperator& UrlOperator::operator() (const QString& key, const char *value)
	{
		return (*this) (key, QString::fromLatin1 (value));
	}

	UrlOperator& UrlOperator::operator() (const QString& key, int value)
	{
		return (*this) (key, QString::number (value));
	}

	UrlOperator& UrlOperator::operator-= (const QString& key)
	{
#if QT_VERSION < 0x050000
		Url_.removeQueryItem (key);
#else
		Query_.removeQueryItem (key);
#endif

		return *this;
	}

	QUrl UrlOperator::operator() ()
	{
		Flush ();
		return Url_;
	}
}
}
