/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 * Copyright (C) 2012       Maxim Ignatenko
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "fdguard.h"

#include <fcntl.h>
#include <unistd.h>

namespace LeechCraft
{
namespace Util
{
	FDGuard::FDGuard (const char *file, int mode)
	: FD_ { open (file, mode) }
	{
	}

	FDGuard::FDGuard (FDGuard&& other)
	: FD_ { other.FD_ }
	{
		other.FD_ = -1;
	}

	FDGuard& FDGuard::operator= (FDGuard&& other)
	{
		swap (*this, other);

		return *this;
	}

	FDGuard::~FDGuard ()
	{
		if (FD_ >= 0)
			close (FD_);
	}

	FDGuard::operator bool () const
	{
		return FD_ >= 0;
	}

	FDGuard::operator int () const
	{
		return FD_;
	}

	void swap (FDGuard& g1, FDGuard& g2)
	{
		std::swap (g1.FD_, g2.FD_);
	}
}
}
