/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include <QStringList>
#include <QtDebug>
#include "tagsfiltermodel.h"

namespace LeechCraft
{
namespace Util
{
	TagsFilterModel::TagsFilterModel (QObject *parent)
	: QSortFilterProxyModel (parent)
	, NormalMode_ (true)
	, Separator_ (";")
	, TagsMode_ (TagsInclusionMode::All)
	{
	}

	void TagsFilterModel::SetSeparator (const QString& separator)
	{
		Separator_ = separator;

		if (dynamicSortFilter ())
			invalidateFilter ();
	}

	void TagsFilterModel::SetTagsInclusionMode (TagsFilterModel::TagsInclusionMode mode)
	{
		TagsMode_ = mode;

		if (dynamicSortFilter ())
			invalidateFilter ();
	}

	void TagsFilterModel::setTagsMode (bool tags)
	{
		NormalMode_ = !tags;

		if (dynamicSortFilter ())
			invalidateFilter ();
	}

	void TagsFilterModel::enableTagsMode ()
	{
		setTagsMode (true);
	}

	void TagsFilterModel::disableTagsMode ()
	{
		setTagsMode (false);
	}

	bool TagsFilterModel::filterAcceptsRow (int source_row, const QModelIndex& index) const
	{
		if (NormalMode_)
		{
			if (index.isValid () && sourceModel ()->rowCount (index))
				return true;

			const auto& pattern = filterRegExp ().pattern ();
			if (pattern.isEmpty ())
				return true;

			for (int i = 0, cc = sourceModel ()->columnCount (index); i < cc; ++i)
			{
				const auto& rowIdx = sourceModel ()->index (source_row, i, index);
				const auto& str = rowIdx.data ().toString ();
				if (str.contains (pattern) || filterRegExp ().exactMatch (str))
					return true;
			}

			return false;
		}
		else
		{
			QStringList filterTags;
			const auto& pattern = filterRegExp ().pattern ();
			for (const auto& s : pattern.split (Separator_, QString::SkipEmptyParts))
				filterTags << s.trimmed ();

			if (!filterTags.size ())
				return true;

			const auto& itemTags = GetTagsForIndex (source_row);
			for (int i = 0; i < filterTags.size (); ++i)
			{
				bool found = false;
				for (int j = 0; j < itemTags.size (); ++j)
					if (itemTags.at (j).contains (filterTags.at (i)))
					{
						found = true;
						break;
					}
				if (!found)
					return false;
				else if (TagsMode_ == TagsInclusionMode::Any)
					return true;
			}
			return true;
		}
	}
}
}
