/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include "xdgconfig.h"

template<typename T>
class QList;

class QStringList;

namespace LeechCraft
{
namespace Util
{
namespace XDG
{
	/** @brief Describes the various types of XDG <code>.desktop</code>
	 * files.
	 */
	enum class Type
	{
		/** @brief Unknown type.
		 */
		Other,

		/** @brief A shortcut to an application.
		 */
		Application,

		/** @brief A shortcut to an URL.
		 */
		URL,

		/** @brief A shortcut to a directory.
		 */
		Dir
	};

	/** @brief Returns a set of typical directories with
	 * <code>desktop</code> files of the given \em types.
	 *
	 * @param[in] types The types of the interesting <code>.desktop</code>
	 * files.
	 * @return The list of the directories where the said files typically
	 * reside.
	 */
	UTIL_XDG_API QStringList ToPaths (const QList<Type>& types);
}
}
}
